package org.bitbucket.bradleysmithllc.etlagent.informatica.client;

import org.bitbucket.bradleysmithllc.etlagent.informatica.dto.*;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.*;
import org.bitbucket.bradleysmithllc.etlunit.InformaticaError;
import org.bitbucket.bradleysmithllc.webserviceshubclient.parameter.ParameterFile;

import java.io.File;
import java.util.List;
import java.util.Map;

public class AgentClient implements InformaticaRepositoryClient, InformaticaIntegrationServiceClient, InformaticaWebServicesHubClient
{
	private final InformaticaAgentClient informaticaAgentClient;

	private final InformaticaIntegrationService informaticaIntegrationService;
	private final InformaticaRepository informaticaRepository;

	public AgentClient(InformaticaRepository informaticaRepository, String clientId) {
		this.informaticaRepository = informaticaRepository;
		informaticaIntegrationService = null;

		informaticaAgentClient = new InformaticaAgentClient(clientId, informaticaRepository.getInformaticaDomain().getAgentHost(), informaticaRepository.getInformaticaDomain().getAgentPort());
	}

	public AgentClient(InformaticaIntegrationService informaticaIntegrationService, String clientId) {
		this.informaticaIntegrationService = informaticaIntegrationService;
		this.informaticaRepository = informaticaIntegrationService.getInformaticaRepository();

		informaticaAgentClient = new InformaticaAgentClient(clientId, informaticaRepository.getInformaticaDomain().getAgentHost(), informaticaRepository.getInformaticaDomain().getAgentPort());
	}

	@Override
	public InformaticaExecutionResult executeWorkflow(String workFlow, String folder, ParameterFile parmFile) throws Exception {
		return executeWorkflowTask(workFlow, null, null, folder, parmFile, null);
	}

	@Override
	public InformaticaExecutionResult executeWorkflow(String workFlow, String folder, ParameterFile parmFile, Map<RemoteFile.fileType, List<RemoteFile>> remoteFiles) throws Exception {
		return executeWorkflowTask(workFlow, null, null, folder, parmFile, remoteFiles);
	}


	@Override
	public InformaticaExecutionResult executeWorkflowTask(String workFlow, String task, String folder, ParameterFile parmFile) throws Exception {
		return executeWorkflowTask(workFlow, null, task, folder, parmFile, null);
	}

	@Override
	public InformaticaExecutionResult executeWorkflowTask(String workFlow, String task, String folder, ParameterFile parmFile, Map<RemoteFile.fileType, List<RemoteFile>> remoteFiles) throws Exception {
		return executeWorkflowTask(workFlow, null, task, folder, parmFile, remoteFiles);
	}

	@Override
	public InformaticaExecutionResult executeWorkflowTask(String workFlow, String runInstanceName, String task, String folder, ParameterFile parmFile) throws Exception {
		return executeWorkflowTask(workFlow, null, task, folder, parmFile, null);
	}

	@Override
	public InformaticaExecutionResult executeWorkflowTask(String workFlow, String runInstanceName, String task, String folder, ParameterFile parmFile, Map<RemoteFile.fileType, List<RemoteFile>> remoteFiles) throws Exception {
		ExecuteWorkflowRequestDTO request = new ExecuteWorkflowRequestDTO(folder, workFlow, task, runInstanceName, parmFile);

		request.setInformaticaDomain(informaticaRepository.getInformaticaDomain().getDomainName());
		request.setInformaticaRepository(informaticaIntegrationService.getInformaticaRepository().getRepositoryName());
		request.setInformaticaRepository(informaticaRepository.getRepositoryName());

		// if files are posted, pass them on
		if (remoteFiles != null)
		{
			for (List<RemoteFile> remoteFileList : remoteFiles.values())
			{
				for (RemoteFile remoteFile : remoteFileList)
				{
					request.addRequestFile(remoteFile);
				}
			}
		}

		return informaticaAgentClient.executeWorkflow(request);
	}

	@Override
	public InformaticaExecutionResult executeWorkflow(String workFlow, String runInstanceName, String folder, ParameterFile parmFile) throws Exception {
		return executeWorkflowTask(workFlow, runInstanceName, null, folder, parmFile);
	}

	@Override
	public InformaticaExecutionResult executeWorkflow(String workFlow, String runInstanceName, String folder, ParameterFile parmFile, Map<RemoteFile.fileType, List<RemoteFile>> remoteFiles) throws Exception {
		return executeWorkflowTask(workFlow, runInstanceName, null, folder, parmFile, remoteFiles);
	}

	@Override
	public InformaticaIntegrationService getIntegrationService() {
		return informaticaIntegrationService;
	}

	@Override
	public void createConnection(String serverName, String databaseName, String databaseUsername, String databasePassword, String relationalConnectionName) throws Exception, InformaticaError {
		CreateConnectionRequestDTO createConnectionRequestDTO = new CreateConnectionRequestDTO(relationalConnectionName, serverName, databaseName, databaseUsername, databasePassword);

		createConnectionRequestDTO.setInformaticaDomain(informaticaRepository.getInformaticaDomain().getDomainName());
		createConnectionRequestDTO.setInformaticaRepository(informaticaRepository.getRepositoryName());

		informaticaAgentClient.createConnection(createConnectionRequestDTO);
	}

	@Override
	public void createFolder(String folder) throws Exception, InformaticaError {
		createFolder(folder, false);
	}

	@Override
	public void createFolder(String folder, boolean temporary) throws Exception, InformaticaError {
		CreateFolderRequestDTO createFolderRequestDTO = new CreateFolderRequestDTO(folder);

		createFolderRequestDTO.setInformaticaDomain(informaticaRepository.getInformaticaDomain().getDomainName());
		createFolderRequestDTO.setInformaticaRepository(informaticaRepository.getRepositoryName());

		informaticaAgentClient.createFolder(createFolderRequestDTO);
	}

	@Override
	public void deleteFolder(String folder) throws Exception, InformaticaError {
		deleteFolder(folder, true);
	}

	@Override
	public void deleteFolder(String folder, boolean failIfNotExists) throws Exception, InformaticaError {
		DeleteFolderRequestDTO deleteFolderRequestDTO = new DeleteFolderRequestDTO(folder, failIfNotExists);

		deleteFolderRequestDTO.setInformaticaDomain(informaticaRepository.getInformaticaDomain().getDomainName());
		deleteFolderRequestDTO.setInformaticaRepository(informaticaRepository.getRepositoryName());

		informaticaAgentClient.deleteFolder(deleteFolderRequestDTO);
	}

	@Override
	public void deleteConnection(String connectionName) throws Exception, InformaticaError {
		deleteConnection(connectionName, true);
	}

	@Override
	public void deleteConnection(String connectionName, boolean failIfNotExists) throws Exception, InformaticaError {
		DeleteConnectionRequestDTO deleteConnectionRequestDTO = new DeleteConnectionRequestDTO(connectionName, failIfNotExists);

		deleteConnectionRequestDTO.setInformaticaDomain(informaticaRepository.getInformaticaDomain().getDomainName());
		deleteConnectionRequestDTO.setInformaticaRepository(informaticaRepository.getRepositoryName());

		informaticaAgentClient.deleteConnection(deleteConnectionRequestDTO);
	}

	@Override
	public void deleteTemporaryFolders() throws Exception, InformaticaError {
		throw new UnsupportedOperationException();
	}

	@Override
	public void exportWorkflowToXml(String workFlow, String folder, File output) throws Exception, InformaticaError {
		ExportWorkflowRequestDTO exportWorkflowRequestDTO = new ExportWorkflowRequestDTO(folder, workFlow, output);

		exportWorkflowRequestDTO.setInformaticaDomain(informaticaRepository.getInformaticaDomain().getDomainName());
		exportWorkflowRequestDTO.setInformaticaRepository(informaticaRepository.getRepositoryName());

		informaticaAgentClient.exportWorkflow(exportWorkflowRequestDTO);
	}

	@Override
	public void exportAllWorkflowsToXml() throws Exception, InformaticaError {
		throw new UnsupportedOperationException();
	}

	@Override
	public void importWorkflowFromXml(String folderPrefix, InformaticaIntegrationService newService, File xmlFile, List<String> flist) throws Exception {
		String name = xmlFile.getName();

		if (name.toLowerCase().endsWith(".xml"))
		{
			name = name.substring(0, name.length() - 4);
		}

		ImportWorkflowRequestDTO importWorkflowRequestDTO = new ImportWorkflowRequestDTO(folderPrefix, name, xmlFile, flist);

		importWorkflowRequestDTO.setInformaticaDomain(informaticaRepository.getInformaticaDomain().getDomainName());
		importWorkflowRequestDTO.setInformaticaRepository(informaticaRepository.getRepositoryName());
		importWorkflowRequestDTO.setInformaticaIntegrationService(newService.getIntegrationServiceName());

		informaticaAgentClient.importWorkflow(importWorkflowRequestDTO);
	}

	@Override
	public void importWorkflowFromXml(String folderPrefix, InformaticaIntegrationService newService, File xml) throws Exception {
		importWorkflowFromXml(folderPrefix, newService, xml, null);
	}

	@Override
	public List<String> listFolders() throws Exception, InformaticaError {
		throw new UnsupportedOperationException();
	}

	@Override
	public int purgeTestFolders() throws Exception {
		PurgeTestFoldersRequestDTO purgeTestFoldersRequestDTO = new PurgeTestFoldersRequestDTO();

		purgeTestFoldersRequestDTO.setInformaticaDomain(informaticaRepository.getInformaticaDomain().getDomainName());
		purgeTestFoldersRequestDTO.setInformaticaRepository(informaticaRepository.getRepositoryName());

		return informaticaAgentClient.purgeTestFolders(purgeTestFoldersRequestDTO);
	}

	@Override
	public InformaticaRepository getInformaticaRepository() {
		return informaticaRepository;
	}

	@Override
	public InformaticaDomain getInformaticaDomain() {
		return informaticaRepository.getInformaticaDomain();
	}

	@Override
	public void connect() throws Exception {
		// nothing required here
	}

	@Override
	public void dispose() {
		// nothing
	}

	@Override
	public InformaticaWebServicesHub getWebServicesHub() {
		return null;  //To change body of implemented methods use File | Settings | File Templates.
	}
}